#!/usr/bin/env bash
#!/bin/env bash

## Name: coverage.sh
## Version: 1.2.1
## Date: Mar 09, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Coverage profile computation for RNAprof


ECHO_E="echo -e"
PREFIX="."
GFFPROF="input_"
MAX=1

usage() {
    $ECHO_E "Usage: ./coverage.sh [OPTIONS]\n"
    $ECHO_E "Optional:"
    $ECHO_E "-C|--CPU MAX"
    $ECHO_E "\tRun up to MAX processes in parallel. By default, MAX=1."
    $ECHO_E "-g|--gffprof"
    $ECHO_E "\tReannotated GFF is used for coverage profile computation. By default, given GFF is used."
    $ECHO_E "-h|-?|--help"
    $ECHO_E "\tPrint this help."
    $ECHO_E "-m|--memory INT"
    $ECHO_E "\tThe maxium required memory for each process, specified either in bytes or with a K, M, or G suffix. By default, 2G."
    $ECHO_E "-p|--prefix PREFIX"
    $ECHO_E "\tThis is the same directory as PREFIX given in configuration step. All generated scripts are located in PREFIX/script. By default,PREFIX=.\n"

    exit 1
}

if [ ! $1 ]
then
    usage
fi

while :
do
    case $1 in
	-h | --help | -\?)
	    usage
	    ;;
	-p | --prefix)
	    PREFIX=$2
	    shift 2
	    ;;
	--prefix=*)
            PREFIX=${1#*=} #delete everything up till '='
            shift
            ;;
	-C | --CPU)
	    MAX=$2
	    shift 2
	    ;;
	--CPU=*)
            MAX=${1#*=}
            shift
            ;;
	-g | --gffprof)
	    GFFPROF="prof_"
	    shift
	    ;;
	-m | --memory)
	    MEM=$2
	    shift 2
	    ;;
	--memory=*)
            MEM=${1#*=}
            shift
            ;;
	-w | --wholeDE)
	    WHOLEDE_OPTION="--wholeDE"
	    shift
	    ;;
	-*)
	    $ECHO_ "Unknown option: $1" >&2
	    shift
	    ;;
	--) #end of all options
	    shift
	    break
	    ;;
	*) #no more options, stop while loop
	    break
	    ;;
    esac
done

cd $PREFIX
PREFIX=$PWD
cd - > /dev/null

rm -f $PREFIX/rnaseq/*_refined_*am

$ECHO_E "\nCOMPUTE COVERAGE PROFILES..."

## run in parallel coverage_*
find $PREFIX/script -name coverage_\* | xargs -I {} --max-procs=$MAX bash -c "bash {} $GFFPROF"

rm -rf $PREFIX/rnaseq/UNS*.tsv $PREFIX/rnaseq/count

if [ $WHOLEDE_OPTION ]
then
    while read line
    do
	DESCARR=(${line//"\t"/ })
	instance=`basename ${DESCARR[1]}`
	cat $PREFIX/rnaseq/htseqcount/${instance}*.cnt > $PREFIX/rnaseq/htseqcount/htseq_${instance}.cnt
	rm $PREFIX/rnaseq/htseqcount/${instance}*.cnt
    done < $PREFIX/rnaseq/tmp_description
fi
rm $PREFIX/rnaseq/tmp_description

## directory containing sources
SRC=$0
SRC=`dirname $SRC`
cd $SRC
SRC=$PWD
cd - > /dev/null

## make configuration files for RNAprof execution
perl $SRC/perl/makeRNAprof.pl --input=$PREFIX/rnaseq/config.tsv --GenomeSize=$PREFIX/genome/genomesize.txt --outputPrefix=$PREFIX/script/rnaprof --path=$PREFIX --src=$SRC --gffprof=$GFFPROF --memory=$MEM
