#!/usr/bin/env perl
#!/bin/env perl

## Name: getbasecovchr.pl
## Version: 1.2.1
## Date: Mar 09, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# compute vectors of coverage for every gene
# work with only ONE chromosome in input file
# --input, input gff/gtf file, containing only studied transcripts
# --count, input count file = output from genomeCoverageBed
# --base, 0- or 1-base coordinates in count file
# --output, .cur output file
 
use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $count;
my $output = "output.cur";
my $base = 0;
my $lineref;
my $linecount;
my @argsref;
my @argscount;
my $i;
my $cbase;
my $start;
my $start_prev;
my $end;
my $end_prev;
my @cov;
my @cov_prev;
my $last_unexp;
my $indexcount_prev;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "count=s" => \$count,
			 "output=s" => \$output,
			 "base=s" => \$base)
	    or defined $help);

open(GFF, $input) or die("GFF file not found\n");
open(COUNT, $count) or die("Count file not found\n");
open(OUT, ">$output");

$cbase = 1 - $base;
$lineref = <GFF>;
$linecount = "0\t-1\t\0";
if ($lineref) {
    chomp($lineref);
}
else {
    print "Empty gff file\n";
    exit 0;
}
@argsref = split("\t", $lineref);
@argscount = split("\t", $linecount);
$start_prev = 0;
$end_prev = 0;
@cov_prev = ();

do {
    @cov = ();
    $start = $argsref[3];
    $end = $argsref[4];

    #< print chromosome and position of transcript
    print OUT "$argsref[0]\t$argsref[3]\t$argsref[4]\t";
    #>
    
    #< ignored part
    while ($linecount && $argscount[1]+$cbase < $argsref[3]) {
	$linecount = <COUNT>;
	if ($linecount) {
	    chomp($linecount);
	    @argscount = split("\t", $linecount);
	}
    }
    #>

    #< common part with previous transcript
    $i = $start;
    while ($i <= $end_prev) {
	push(@cov, $cov_prev[$i-$start_prev]);
	$i++;
    }
    #>

    #< unexpressed part following the common part
    if ($linecount) {
	if ($end <= $argscount[1]+$cbase-1) {
	    $last_unexp = $end;
	    $indexcount_prev = $end + 1;
	}
	else {
	    $last_unexp = $argscount[1]+$cbase-1;
	}
    }
    else {
	$last_unexp = $end;
	$indexcount_prev = $end + 1;
    }
    while ($i <= $last_unexp) {
	push(@cov, 0);
	$i++;
    }
    #>
    
    #< expressed parts
    while ($linecount && $argscount[1]+$cbase <= $end) {
	push(@cov, $argscount[2]);
	$i = $argscount[1]+1;
	$indexcount_prev = $argscount[1]+$cbase;
	$linecount = <COUNT>;
	if ($linecount) {
	    chomp($linecount);
	    @argscount = split("\t", $linecount);
	    if ($argscount[1]+$cbase <= $end) {
		#< unexpressed parts in between
		while ($i < $argscount[1]) {
		    push(@cov, 0);
		    $i++;
		}
		#>
	    }
	}
    }
    #>
    
    #< ending unexpressed part
    $i = $indexcount_prev + 1;
    while ($i <= $end) {
	push(@cov, 0);
	$i++;
    }
    #>
    
    #< print
    for ($i=0; $i<=$#cov; $i++) {
	print OUT "$cov[$i]\t";
    }
    print OUT "\n";
    #>

    @cov_prev = @cov;
    $start_prev = $start;
    $end_prev = $end;

    $lineref = <GFF>;
    if ($lineref) {
	chomp($lineref);
	@argsref = split("\t", $lineref);
    }
} until (!$lineref);

close(GFF);
close(COUNT);
close(OUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl getbasecovchr.pl --input GFF --count COUNT [--base 0/1] [--output OUT] [--help|?] \n";
    print "Notice:\n";
    print "- COUNT is 0-based by default\n";
    print "- GFF only contains one chromosome\n";
    exit 1;
}
