#!/usr/bin/env perl
#!/bin/env perl

## Name: getsplicedtsc.pl
## Version: 1.2.1
## Date: Mar 09, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# get gff file with spliced genes, containing genes and their exons
# --input, unspliced segments = output of segment.pl with nosplit
# --junction, intron list = output of fusionintron.pl
# --strand, strand +/- of transcripts
# --prefix, gene prefix
# --min, minimum number of mapped reads to take into account a transcript
# --output, output gff file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $junction;
my $strand = ".";
my $output = "output.gff";
my $prefix = "TSC";
my $minread = 1;
my $line;
my $lineno;
my @args;
my @argsno;
my $count;
my $exonstart;
my $exonend;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "junction=s" => \$junction,
			 "strand=s" => \$strand,
			 "prefix=s" => \$prefix,
			 "min=i" => \$minread,
			 "output=s" => \$output)
	    or defined $help);

open(NOSPLIT, $input) or die("Input file not found\n");
open(INTRON, $junction) or die("Intron list not found\n");
open(OUT, ">$output");

#< gene id
$count = 0;
#>

$line = <INTRON>;
if ($line) {
    chomp($line);
    @args = split("\t", $line);
}

while ($lineno=<NOSPLIT>) {
    $count++;
    chomp($lineno);
    @argsno = split("\t", $lineno);
    $exonstart = $argsno[1] + 1;
    
    #< skip introns coming before gene
    while ($line && ($args[1] <= $argsno[1]) && ($args[0] eq $argsno[0])) {
	$line = <INTRON>;
	if ($line) {
	    chomp($line);
	    @args = split("\t", $line);
	}
    }
    #>
    if ($argsno[3] >= $minread) {
	#< print gene
	print OUT "$argsno[0]\tGFFPROF\tgene\t$exonstart\t$argsno[2]\t.\t$strand\t.\tID=${prefix}_${count};Name=${prefix}_${count}\n";
	#>
    
	while (($line) && ($args[2]<$argsno[2]) && ($args[0] eq $argsno[0])) {
	    $exonend = $args[1];
	    #< print exon
	    print OUT "$argsno[0]\tGFFPROF\texon\t$exonstart\t$exonend\t.\t$strand\t.\tParent=${prefix}_${count}\n";
	    #>
	    $exonstart = $args[2] + 1;
	    $line = <INTRON>;
	    if ($line) {
		chomp($line);
		@args = split("\t", $line);
	    }
	}
	
	$exonend = $argsno[2];
	#< print the last exon
	print OUT "$argsno[0]\tGFFPROF\texon\t$exonstart\t$exonend\t.\t$strand\t.\tParent=${prefix}_${count}\n";
	#>
    }
    else {
	while (($line) && ($args[2]<$argsno[2]) && ($args[0] eq $argsno[0])) {
	    $line = <INTRON>;
            if ($line) {
                chomp($line);
                @args = split("\t", $line);
            }
	}
    }
}

close(INTRON);
close(NOSPLIT);
close(OUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl getsplicedtsc.pl --input SEGMENT --junction INTRON [--min MIN] [--strand +/-] [--prefix PRE] [--output GFF] [--help|?] \n";
    exit 1;
}
